import json
from pathlib import Path


class Config:
    __GROUP_URL_KEY = "group_url"
    __PYVKO_CONFIG_KEY = "pyvko_config"

    def __init__(self, json_object: dict) -> None:
        super().__init__()

        self.__group_url = json_object[Config.__GROUP_URL_KEY]
        self.__pyvko_config = Path(json_object[Config.__PYVKO_CONFIG_KEY])

    @property
    def group_url(self) -> str:
        return self.__group_url

    @property
    def pyvko_config_path(self) -> Path:
        return self.__pyvko_config

    @staticmethod
    def read(path: Path) -> 'Config':
        print(path.absolute())

        with path.open() as config_file:
            config_object = json.load(config_file)

            return Config(config_object)

    def save(self, path: Path):
        json_object = {
            Config.__GROUP_URL_KEY: self.group_url,
            Config.__PYVKO_CONFIG_KEY: str(self.pyvko_config_path)
        }

        with path.open("w") as config_file:
            json.dump(json_object, config_file, indent=4)



GROUP_CATEGORIES = [
    'academ',
    'artchaos',
    'countryside',
    'moments',
    'nanoreports',
    'nsu',
    'patterns',
    'reports',
    'series',
    '_meta',
    'sessions',
    'piter2017',
    'poll',
]

CLOSED_NAMES = [
    'annette',
    'cafe',
    'photoclub',
    'private',
    'sessions',
    'technologika',
    'fit',
    'fen',
    'shabanov',
    'lika',
    'luci',
    'leonova',
    'palliative'
]

OUR_PEOPLE_NAMES = [
    'absaidulieva',
    'ahmary',
    'anischenko',
    'annette',
    'aziza',
    'babich',
    'babicheva',
    'banan',
    'belovezhets',
    'biryukov',
    'biserova',
    'budaeva',
    'cafe',
    'contest',
    'damdinov',
    'dubynina',
    'ermishina',
    'esler',
    'faina',
    'fokina',
    'gapenko',
    'gassan',
    'golodyaeva',
    'guzhvina',
    'iokimaru',
    'kachanova',
    'kamaeva',
    'karmanova',
    'kolpakov',
    'korobova',
    'kosikhin',
    'leonova',
    'lika',
    'lina',
    'me',
    'melgunov',
    'miroshnichenko',
    'nam',
    'nastya_med',
    'nigga',
    'okhvat',
    'oydopova',
    'pesterev',
    'porechnaya_ggf',
    'photoclub',
    'rats',
    'sazhina',
    'sharipova',
    'sher',
    'shlee',
    'shmakov',
    'spice',
    'toropova',
    'vesnina',
    'yartseva',
    'zimbickiy'
]

TEST_OUR_PEOPLE_NAMES = [
    'absaidulieva',
    'ahmary',
    'anischenko',
    'annette',
    'aziza',
    'babich',
    'babicheva',
    'banan',
    'belovezhets',
    'biryukov',
    'biserova',
    'budaeva',
    'cafe',
    'contest',
    'damdinov',
    'dubynina',
    'ermishina',
    'esler',
    'faina',
    'fokina',
    'gapenko',
    'gassan',
    'golodyaeva',
    'guzhvina',
    'iokimaru',
    'kachanova',
    'kamaeva',
    'karmanova',
    'kolpakov',
    'korobova',
    'kosikhin',
    'leonova',
    'lika',
    'lina',
    'me',
    'melgunov',
    'miroshnichenko',
    'nam',
    'nastya_med',
    'nigga',
    'okhvat',
    'oydopova',
    'pesterev',
    'porechnaya_ggf',
    'photoclub',
    'rats',
    'sazhina',
    'sharipova',
    'sher',
    'shlee',
    'shmakov',
    'spice',
    'toropova',
    'vesnina',
    'yartseva',
    'zimbickiy'
]

diff = list(set(TEST_OUR_PEOPLE_NAMES).difference(OUR_PEOPLE_NAMES))

print(diff)

a = 7

STAGES_NAMES = [
    'stage0.gif',
    'stage0.recapture',
    'stage1.filter',
    'stage2.develop',
    'stage2.ourate',
    'stage3.ready',
    'stage4.published',
]



import re
from pathlib import Path
from typing import List

from v3_0.shared import config
from v3_0.shared.helpers.parting_helper import PartingHelper

SETS_KEY = "varied_folders"
PARTS_KEY = "parts"
FILES_KEY = "files"

__SETS = {
    SETS_KEY: {},
}

__PARTS = {
    PARTS_KEY: {},
}

STANDALONE_FILE = "file"

__FILES = {
    FILES_KEY: {},
}

FILES = __FILES

__PARTS_AND_FILES = {
    **__PARTS,
    **__FILES
}


def __categories(names):
    return {name: __SETS for name in names}


__disk_structure = {
    '_meta': STANDALONE_FILE,
    'instagram': __SETS,
    'justin': __categories(config.GROUP_CATEGORIES),
    'photoclub': __SETS,
    'closed': __categories(config.CLOSED_NAMES),
    # 'selections': __SETS,  # only jpegs. should it be sets?
    'unpublished': {
        **{
            "dad": __SETS,
            "mom": __SETS,
        },
        **__SETS,
    },
    'stages': __categories(config.STAGES_NAMES),
    'watermarks': __SETS  # sets?
}

_photoset_structure = {
    'instagram': __FILES,
    'justin': {
        'academ': __PARTS_AND_FILES,
        'artchaos': __PARTS_AND_FILES,
        'countryside': __PARTS_AND_FILES,
        'moments': __PARTS_AND_FILES,
        'nsu': __PARTS_AND_FILES,
        'patterns': __PARTS_AND_FILES,
        'nanoreports': __FILES,
        'reports': FILES,
        'series': __FILES,
        '_meta': __FILES,
        'session': __FILES,
        'piter2017': __PARTS
    },
    'photoclub': __FILES,
    'closed': {name: __FILES for name in config.CLOSED_NAMES},
    'our_people': {name: __FILES for name in config.OUR_PEOPLE_NAMES},
    "gif": __PARTS_AND_FILES,
    "selection": __FILES,
    "preview": __FILES,
    "ondemand": __FILES,
}

_photoset_structure.update(__PARTS_AND_FILES)

__photos_folder = "photos"
METAFILE_NAME = "_meta"


class Structure:
    __FLAG_KEYS = [
        SETS_KEY,
        FILES_KEY,
        PARTS_KEY,
    ]

    __PHOTOSET_NAME_REGEXP = "\d\d\.\d\d\.\d\d\.\w+"

    def __init__(self, name: str, description: dict, path: Path) -> None:
        super().__init__()

        self.__name = name
        self.__substructures = {}
        self.__files = []
        self.__path = path

        for subname, subdesc in description.items():
            if subdesc == STANDALONE_FILE:
                self.__files.append(subname)
            elif subname in Structure.__FLAG_KEYS:
                pass
            else:
                self.__substructures[subname] = Structure(subname, subdesc, self.__path / subname)

        self.__has_implicit_sets = SETS_KEY in description
        self.__has_files = FILES_KEY in description
        self.__has_parts = PARTS_KEY in description

    @property
    def name(self) -> str:
        return self.__name

    @property
    def path(self) -> Path:
        return self.__path

    @property
    def has_substructures(self) -> bool:
        return len(self.substructures) > 0

    @property
    def substructures(self) -> List['Structure']:
        return list(self.__substructures.values())

    @property
    def has_implicit_sets(self) -> bool:
        return self.__has_implicit_sets

    @property
    def has_unlimited_files(self) -> bool:
        return self.__has_files

    @property
    def has_parts(self) -> bool:
        return self.__has_parts

    def __str__(self) -> str:
        return self.__name

    def has_file(self, item: str) -> bool:
        return self.__has_files or item in self.__files

    def has_substructure(self, item: str) -> bool:
        if item in self.__substructures:
            return True
        elif self.has_parts:
            return PartingHelper.is_part_name(item)
        else:
            return False

    def has_set(self, item: str) -> bool:
        return not self.has_substructure(item) and self.has_implicit_sets

    def __getitem__(self, key) -> 'Structure':
        if key in self.__substructures:
            return self.__substructures[key]
        elif self.__has_implicit_sets and re.match(Structure.__PHOTOSET_NAME_REGEXP, key) is not None:
            return Structure(key, _photoset_structure, self.path / key)
        else:
            assert False


disk_structure = Structure("disk", __disk_structure, Path(""))
photoset_structure = Structure("photoset", _photoset_structure, Path(""))



from pathlib import Path

from pyvko.pyvko import Pyvko

from v3_0.shared.configuration.config import Config
from v3_0.args import Args
from v3_0.shared.models.world import World
from v3_0.shared.singleton import Singleton


class Configurator(Singleton):
    __CONFIG_PATH = "configs/config.json"

    # noinspection PyMethodMayBeStatic
    def get_args(self) -> Args:
        config = Config.read(Path(Configurator.__CONFIG_PATH))

        pyvko = Pyvko(config.pyvko_config_path)

        group = pyvko.get_group(config.group_url)
        photos_uploader = pyvko.get_photos_uploader()

        world = World()

        args = Args(world=world, group=group, uploader=photos_uploader)

        return args



from setuptools import setup, find_packages

setup(
    name='justin',
    version='0.0.0',
    packages=find_packages(),
    install_requires=[
        'pyvko',
        'Pillow',
    ],
    url='',
    license='MIT',
    author='Harry Djachenko',
    author_email='i.s.djachenko@gmail.com',
    description=''
)



import argparse
from functools import lru_cache

from pyvko.group.group import Group
from pyvko.photos.photos_uploader import PhotosUploader

from v3_0.shared.models.world import World


class Args(argparse.Namespace):
    @staticmethod
    @lru_cache()
    def instance() -> 'Args':
        # pyvko = Pyvko()

        return Args(World.instance())

    def __init__(self, world: World, group: Group = None, uploader: PhotosUploader = None) -> None:
        super().__init__()

        self.__world = world
        self.__group = group
        self.__uploader = uploader

    @property
    def world(self) -> World:
        return self.__world

    @property
    def group(self) -> Group:
        return self.__group

    @property
    def uploader(self) -> PhotosUploader:
        return self.__uploader



from abc import ABC
from functools import lru_cache
from typing import TypeVar, Type

T = TypeVar('T', bound='Singleton')


class Singleton(ABC):
    @classmethod
    @lru_cache()
    def instance(cls: Type[T]) -> T:
        return cls()
